/*
 * error.c - Error reporting
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "util.h"
#include "error.h"


static struct file_name {
    const char *name;
    struct file_name *next;
} *file_names = NULL;

static const struct location *curr = NULL;


void clerrorf(const char *fmt,...)
{
    va_list ap;

    if (curr) fprintf(stderr,"%s:%d: ",curr->file,curr->line);
    va_start(ap,fmt);
    vfprintf(stderr,fmt,ap);
    va_end(ap);
    fputc('\n',stderr);
    exit(1);
}


void clperror(const char *s)
{
    clerrorf("%s: %s",s,strerror(errno));
}


void set_location(const struct location *loc)
{
    curr = loc;
}


struct location current_location(void)
{
    struct location loc;

    /*
     * cpp 3.x calls stdin "<stdin>", cpp 2.x calls it "". We translate "" to
     * "<stdin>" here, so that regression tests work with either version.
     */
    loc.file = file_names->name && *file_names->name ?
      file_names->name : "<stdin>";
    loc.line = lineno;
    return loc;
}


void input_file(const char *name)
{
    struct file_name *new;

    new = alloc_t(struct file_name);
    new->name = stralloc(name);
    new->next = file_names;
    file_names = new;
}


void location_cleanup(void)
{
    while (file_names) {
	struct file_name *next = file_names->next;

	free((char *) file_names->name);
	free(file_names);
	file_names = next;
    }
}
