/*
 * ptrace.h - Ptrace wrapper
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef PTRACE_H
#define PTRACE_H

#include <inttypes.h>
#include <sys/types.h>
#include <sys/user.h>


struct pt_break {
    unsigned long addr;		/* breakpoint location */
    uint8_t covered;		/* original opcode */
    int ref;			/* reference count */
    struct pt_break *next;
};

struct pt_process {
    pid_t pid;
    int running;		/* 0 if stopped, exited, or signaled;
				   -1 if starting; 1 if running */
    int terminated;		/* 1 if terminated */
    struct pt_break *bps;	/* breakpoints */
    struct pt_break *stopped;	/* current breakpoint; NULL if none */
    struct user_regs_struct regs;
    int status;			/* valid only if terminated */
    struct pt_process *next;
};


struct pt_process *pt_trace(pid_t pid);
struct pt_process *pt_fork(void);
void pt_terminate(struct pt_process *proc); /* kill and/or reap */

void pt_read(struct pt_process *proc,unsigned long addr,void *buf,size_t size);
void pt_write(struct pt_process *proc,unsigned long addr,const void *buf,
  size_t size);

void pt_run(struct pt_process *proc);
void pt_break(struct pt_process *proc,unsigned long addr);
void pt_unbreak(struct pt_process *proc,unsigned long addr);

/*
 * Returns pointer to descriptor of stopped process. If proc->stopped is NULL,
 * the process has terminated, and its status is stored in proc->status. Note
 * that it is still necessary to call pt_terminate.
 */

struct pt_process *pt_wait(int block);

#endif /* PTRACE_H */
