/*
 * sbug.c - Scriptable deBUGger
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "dwarfx.h"

#include "util.h"
#include "stmt.h"
#include "cpp.h"
#include "sbug.h"


extern int yyparse(void);
extern struct stmt *script;
extern const char *default_script;

extern int debug;
int sbug = 0;

static int verbose = 0;


/* ----- Argument processing ----------------------------------------------- */


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s [-d] [-v] [cpp_option ...] [script_file]\n",
      name);
    fprintf(stderr,"  -d  enable debugging output\n");
    fprintf(stderr,"  -v  verbose operation\n");
    fprintf(stderr,"  cpp_option  -Idir, -Dname[=value], or -Uname\n");
    exit(1);
}


int sbug_main(int argc,char **argv)
{
    static struct stmt exit_0;
    int c;

    sbug = 1;
    while ((c = getopt(argc,argv,"dvD:U:I:")) != EOF)
	switch (c) {
	    static char opt[3] = "-?";

	    case 'd':
		debug = 1;
		break;
	    case 'v':
		verbose = 1;
		break;
	    case 'D':
	    case 'U':
	    case 'I':
		opt[1] = c;
		add_cpp_arg(opt);
		add_cpp_arg(optarg);
		break;
	    default:
		usage(*argv);
	}
    if (argc != optind && argc != optind+1) usage(*argv);
    if (verbose)
	fprintf(stderr,"Loading symbols from file %s\n",argv[optind]);
    init_types();
    if (argc == optind || !strcmp(argv[optind],"-")) {
	if (verbose) fprintf(stderr,"Executing script from standard input\n");
	run_cpp_on_file(NULL);
    }
    else {
	if (verbose)
	    fprintf(stderr,"Executing script from file %s\n",argv[optind]);
	run_cpp_on_file(argv[optind]);
    }
    (void) yyparse();
    reap_cpp();
    execute(script);
    exit_0.u.exit.expr = NULL;
    stmt_exit(&exit_0);
    return 0; /* not reached */
}
