/*
 * sbug.h - Debugger functions
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef SBUG_H
#define SBUG_H

#include <inttypes.h>

#include "dwarfx.h"
#include "ptrace.h"

#include "data.h"
#include "op.h"
#include "stmt.h"


extern int sbug;


struct process;
struct breakpoint;
struct object;


/* ----- data.h ------------------------------------------------------------ */


extern struct dwx_type process_type;
extern struct dwx_type breakpoint_type;
extern struct dwx_type object_type;

struct value *process_value(struct process *process);
struct process *__value_process(struct value *value);

struct value *breakpoint_value(struct breakpoint *breakpoint);
struct breakpoint *__value_breakpoint(struct value *value);

struct value *object_value(struct object *obj);
const struct object *__value_object(struct value *value);

int sbug_put(struct value *value);

int sbug_value_int(struct value *value,uint32_t *res);


/* ----- op.h -------------------------------------------------------------- */


struct value *op_run(const struct op *self);
struct value *op_breakpoint(const struct op *self);
struct value *op_object(const struct op *self);

int destroy_sbug_op(const struct op *stmt);

int sbug_op_index(const struct op *self,struct value **res);
int sbug_op_member(const struct op *self,struct value **res);

struct value *op_object_unit(const struct op *self);

struct value *op_current_bp(const struct op *self);

int sbug_op_eq(struct value *a,struct value *b,struct value **res);
int sbug_op_ne(struct value *a,struct value *b,struct value **res);

struct value *op_address(const struct op *self);
struct value *op_dereference(const struct op *self);
struct value *op_ptr(const struct op *self);

int sbug_op_add(struct value *a,struct value *b,struct value **res);
int sbug_op_sub(struct value *a,struct value *b,struct value **res);

struct value *sbug_assign_hook(struct value *value);

int sbug_op_sizeof_expr(struct value *value,struct value **res);


/* ----- stmt.h ------------------------------------------------------------ */


void stmt_continue_process(const struct stmt *self);

int destroy_sbug_stmt(const struct stmt *stmt);


/* ----- Data access ------------------------------------------------------- */


struct value *dereference_object(struct value *value,int to_data);


/* ----- Other functions --------------------------------------------------- */

int sbug_main(int argc,char **argv);

#endif /* SBUG_H */
