%{
/*
 * script.l - Dump script language
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "util.h"
#include "data.h"
#include "error.h"
#include "sbug.h"

#include "y.tab.h"


int lineno = 1;


static char *dequote(char *s)
{
    if (*s != '"') return s;
    *strrchr(s,'"') = 0;
    return s+1;
}


static void sbug_only(void)
{
    if (!sbug) yyerror("syntax error");
}


%}

/* identifier */
id				[a-zA-Z_][a-zA-Z_0-9]*

%%

unsigned			return TOK_UNSIGNED;
signed				return TOK_SIGNED;
void				return TOK_VOID;
char				return TOK_CHAR;
short				return TOK_SHORT;
int				return TOK_INT;
long				return TOK_LONG;
int8_t				return TOK_INT8_T;
uint8_t				return TOK_UINT8_T;
int16_t				return TOK_INT16_T;
uint16_t			return TOK_UINT16_T;
int32_t				return TOK_INT32_T;
uint32_t			return TOK_UINT32_T;
int64_t				return TOK_INT64_T;
uint64_t			return TOK_UINT64_T;
struct				return TOK_STRUCT;
union				return TOK_UNION;
enum				return TOK_ENUM;

print				return TOK_PRINT;
printf				return TOK_PRINTF;
without				return TOK_WITHOUT;
if				return TOK_IF;
else				return TOK_ELSE;
while				return TOK_WHILE;
for				return TOK_FOR;
break				return TOK_BREAK;
continue			return TOK_CONTINUE;
exit				return TOK_EXIT;
die				return TOK_DIE;
undef				return TOK_UNDEF;

run				return TOK_RUN;
"->"				return PTR_OP;

"||"				return TOK_OR;
&&				return TOK_AND;
"<<"				return TOK_SHL;
">>"				return TOK_SHR;
==				return TOK_EQ;
!=				return TOK_NE;
"<="				return TOK_LE;
">="				return TOK_GE;
"++"				return TOK_INC;
--				return TOK_DEC;

">>="				return SHR_ASSIGN;
"<<="				return SHL_ASSIGN;
"+="				return ADD_ASSIGN;
"-="				return SUB_ASSIGN;
"*="				return MUL_ASSIGN;
"/="				return DIV_ASSIGN;
"%="				return MOD_ASSIGN;
"&="				return AND_ASSIGN;
"^="				return XOR_ASSIGN;
"|="				return OR_ASSIGN;

sizeof				return TOK_SIZEOF;
read				return TOK_READ;
sprintf				return TOK_SPRINTF;

\\?{id}				{ char *id = *yytext == '\\' ? yytext+1 :
				    yytext;
				  int value;
				  switch (dwx_lookup_enum(id,&value)) {
				 	case 0:
					    break;
					case 1:
					    yylval.num = value;
					    return ENUM;
					case 2:
					    yyerror("conflicting values");
					default:
					abort();
				  }
				  yylval.str = stralloc(id);
				  return dwx_lookup_type(dwx_typedef,id,-1,0) ?
				    TYPEDEF_NAME : WORD; }
0[Xx][0-9A-Za-z]+|0[0-7]+|[0-9]+ { char *end;
				  yylval.num = strtoul(yytext,&end,0);
				  if (*end) yyerror("invalid digit in number");
				  return NUMBER; }
'[^']'				{ yylval.num = yytext[1];
				  return NUMBER; }
'\\''				{ yylval.num = '\'';
				  return NUMBER; }
'\\[0-7][0-7]?[0-7]?'		{ yylval.num = strtoul(yytext+2,NULL,8);
				  if (yylval.num > 255)
					yyerror("character value out of range");
				  if (yylval.num & 0x80) yylval.num -= 256;
				  return NUMBER; }
\"[^\"\n\t]*\"			{ *strrchr(yytext,'"') = 0;
				  yylval.str = stralloc(yytext+1);
				  return STRING; }
\$[a-zA-Z_][a-zA-Z_0-9]*	{ yylval.var = var_lookup(yytext+1);
				  return VARIABLE; }
\$\$				{ sbug_only();
				  yylval.var = var_lookup(yytext+1);
				  return VARIABLE; }
\$!				{ sbug_only();
				  return DOLLAR_BANG; }

[\t ]*
\n				lineno++;

^#\ [0-9]+\ \"[^"]*\"(\ [0-9]+)*\n {
				  char *start = strchr(yytext,'"')+1;
                                  *strchr(start,'"') = 0;
				  input_file(start);
				  lineno = strtol(yytext+2,NULL,0); }

.				return *yytext;

%%


void yyerror(const char *s)
{
    fprintf(stderr,"%d: %s near \"%s\"\n",lineno,s,yytext);
    exit(1);
}
