/*
 * sprintf.c - Common code for printf and sprintf
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

#include "data.h"
#include "op.h"
#include "sprintf_generic.h"
#include "sprintf.h"


struct dsc {
    struct arg_list *curr;
    struct arg_list *list;
};


static void check_arg(const struct dsc *d)
{
    if (!d->curr) {
	fprintf(stderr,"not enough arguments for format string\n");
	exit(1);
    }
}


static void first_arg(void *dsc)
{
    struct dsc *d = dsc;

    d->curr = d->list;
}


static void next_arg(void *dsc)
{
    struct dsc *d = dsc;

    check_arg(d);
    d->curr = d->curr->next;
}


static int got_arg(void *dsc)
{
    struct dsc *d = dsc;

    return !!d->curr;
}


static unsigned int get_unum(void *dsc)
{
    struct dsc *d = dsc;

    check_arg(d);
    if (!d->curr->value) {
	d->curr->value = evaluate_lvalue(d->curr->expr);
	if (d->curr->value && d->curr->value->type == &lvalue_type)
	    d->curr->value = dereference_lvalue(d->curr->value);
    }
    return value_int(get(d->curr->value));
}


static int get_snum(void *dsc)
{
    struct dsc *d = dsc;
    int v,bits;

    v = get_unum(dsc);
    if (!d->curr->value) return 0;
    bits = d->curr->value->type->size*8;
    if (bits != sizeof(int)*8)
	if (v & (1 << (bits-1))) v |= ~0 << bits;
    return v;
}


static double get_float(void *dsc)
{
abort();
}


static const char *get_string(void *dsc)
{
    struct dsc *d = dsc;

    check_arg(d);
    if (!d->curr->value) d->curr->value = evaluate(d->curr->expr);
    return value_string(get(d->curr->value));
}


static void errorf(void *dsc,const char *msg,...)
{
    va_list ap;

    va_start(ap,msg);
    vfprintf(stderr,msg,ap);
    va_end(ap);
    exit(1);
}


static void warnf(void *dsc,const char *msg,...)
{
    va_list ap;

    va_start(ap,msg);
    fprintf(stderr,"warning: ");
    vfprintf(stderr,msg,ap);
    va_end(ap);
}


static struct sprintf_ops ops = {
    .first_arg = first_arg,
    .next_arg = next_arg,
    .got_arg = got_arg,
    .get_sint = get_snum,
    .get_uint = get_unum,
    .get_float = get_float,
    .get_string = get_string,
    .errorf = errorf,
    .warnf = warnf
};


char *common_sprintf(const struct op *fmt,struct arg_list *args)
{
    struct value *value;
    struct dsc dsc;
    struct arg_list *walk;
    char *res;

    dsc.list = args;
    value = evaluate(fmt);
    res = sprintf_generic(__value_string(value),&ops,&dsc);
    put(value);
    for (walk = dsc.list; walk; walk = walk->next) {
	if (walk->value) put(walk->value);
	walk->value = NULL;
    }
    return res;
}
