/*
 * stmt.h - Statement construction and execution
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef STMT_H
#define STMT_H

#include "dwarfx.h"

#include "op.h"
#include "var.h"
#include "error.h"


struct without {
    struct op *expr;
    struct without *next;
};

struct stmt {
    void (*fn)(const struct stmt *self);
    union {
	struct {
	    struct op *expr;
	} expr; /* stmt_expression and stmt_undef */
	struct {
	    struct arg_list *args;
	    struct without *without;
	} print;
	struct {
	    struct op *fmt;
	    struct arg_list *args;
	} printf;
	struct {
	    struct op *expr;
	    struct stmt *true;
	    struct stmt *false;
	} if_stmt;
	struct {
	    const struct stmt *loop;
		/* before fixup: points to beginning of loop */
		/* after fixup: points to statement after loop */
	} jump_stmt;
	struct {
	    struct op *expr;
	    struct stmt *body;
	} while_stmt;
	struct {
	    struct op *init;
	    struct op *test;
	    struct op *next;
	    struct stmt *body;
	    struct stmt *next_stmt; /* set when patching the pointers */
	} for_stmt;
	struct {
	    struct op *expr; /* NULL if none */
	} exit;
	struct {
	    struct op *expr; /* NULL if none */
	} die;
	struct {
	    struct op *process;
	} process;
	const char *str;
    } u;
    struct location loc;
    struct stmt *next;	/* next in execution */
    struct stmt *more;	/* next in allocation */
};


struct stmt *new_stmt(void (*fn)(const struct stmt *self));

void fix_statements(struct stmt **stmt);

void stmt_expression(const struct stmt *self);
void stmt_undef(const struct stmt *self);
void stmt_print(const struct stmt *self);
void stmt_printf(const struct stmt *self);
void stmt_if(const struct stmt *self);
void stmt_break(const struct stmt *self);
void stmt_continue(const struct stmt *self);
void stmt_exit(const struct stmt *self);
void stmt_die(const struct stmt *self);
void stmt_while(const struct stmt *self);
void stmt_for(const struct stmt *self);

void execute(struct stmt *pgm);

void stmt_cleanup(void);

#endif /* STMT_H */
