/*
 * util.c - Utility functions
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */


#include <stdlib.h>
#include <stdio.h>

#include "util.h"
#undef free


#ifdef FIND_MEMORY_LEAKS


static struct mem {
    const void *p;
    int size;
    const char *file;
    int line;
    const char *function;
    struct mem *next;
} *mem = NULL;


void register_alloc(const void *p,int size,const char *file,int line,
  const char *function)
{
    struct mem *new;

    new = malloc(sizeof(struct mem));
    if (!new) {
	perror("malloc");
	exit(1);
    }
    new->p = p;
    new->size = size;
    new->file = file;
    new->line = line;
    new->function = function;
    new->next = mem;
    mem = new;
}


void register_free(const void *p)
{
    struct mem **walk;

    for (walk = &mem; *walk; walk = &(*walk)->next)
	if ((*walk)->p == p) {
	    struct mem *this = *walk;

	    *walk = this->next;
	    free(this);
	    return;
	}
    fprintf(stderr,"trying to free unallocated memory at 0x%p\n",p);
    abort();
}


void report_memory_leaks(void)
{
    struct mem *walk;

    for (walk = mem; walk; walk = walk->next)
	fprintf(stderr,"0x%p (%d) %s:%d, %s\n",walk->p,walk->size,walk->file,
	  walk->line,walk->function);
    if (mem) exit(1);
}

#endif /* FIND_MEMORY_LEAKS */
