/*
 * util.h - Utility functions
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef UTIL_H
#define UTIL_H

#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#ifdef FIND_MEMORY_LEAKS

void register_alloc(const void *p,int size,const char *file,int line,
  const char *function);
void register_free(const void *p);
#define free(p) do { register_free(p); free(p); } while (0)
#define realloc(p,size) \
  ({ void *new; if (p) register_free(p); \
    new = realloc(p,size); \
    if (new) register_alloc(new,size,__FILE__,__LINE__,__FUNCTION__); \
    new; })
void report_memory_leaks(void);

#else /* FIND_MEMORY_LEAKS */

#define register_alloc(p,size,file,line,function)
#define report_memory_leaks()

#endif /* !FIND_MEMORY_LEAKS */


#define stralloc(s) ({ char *tmp = strdup(s); \
    if (!tmp) { perror("strdup"); exit(1); }; \
    register_alloc(tmp,strlen(tmp)+1,__FILE__,__LINE__,__FUNCTION__); \
    tmp; })

#define alloc(size) ({ void *tmp = malloc(size); \
    if (!tmp) { perror("malloc"); exit(1); }; \
    register_alloc(tmp,size,__FILE__,__LINE__,__FUNCTION__); \
    tmp; })

#define alloc_t(t) ((t *) alloc(sizeof(t)))

#endif /* UTIL_H */
