/*
 * dwarf.h - Definitions for DWARF2 format, and helper functions
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */



#ifndef DWARF_H
#define DWARF_H

#include "dwarfconst.h"


/* ----- Value decoder ----------------------------------------------------- */


void *dwarf_decode_addr(enum DW_FORM form,const void *p);
const void *dwarf_decode_block(enum DW_FORM form,const void *p,int *length);
int dwarf_decode_sconst(enum DW_FORM form,const void *p);
unsigned dwarf_decode_uconst(enum DW_FORM form,const void *p);
int dwarf_decode_flag(enum DW_FORM form,const void *p);
const void *dwarf_decode_reference(enum DW_FORM form,const void *p);
const char *dwarf_decode_string(enum DW_FORM form,const void *p);
int dwarf_decode_flag(enum DW_FORM form,const void *p);


/* ----- DWARF attribute retrieval ----------------------------------------- */


#define dwarf_attr_addr(a)	dwarf_decode_addr((a)->form,(a)->pos)
#define dwarf_attr_block(a,len)	dwarf_decode_block((a)->form,(a)->pos,len)
#define dwarf_attr_sconst(a)	dwarf_decode_sconst((a)->form,(a)->pos)
#define dwarf_attr_uconst(a)	dwarf_decode_uconst((a)->form,(a)->pos)
#define dwarf_attr_reference(a)	dwarf_decode_reference((a)->form,(a)->pos)
#define dwarf_attr_string(a)	dwarf_decode_string((a)->form,(a)->pos)
#define dwarf_attr_flag(a)	dwarf_decode_flag((a)->form,(a)->pos)


/* ----- DWARF info structure decoder -------------------------------------- */


struct dwarf_attr {
    enum DW_AT attr;
    enum DW_FORM form;
    const void *pos;
    const struct dwarf_attr *next;
};


extern int dwarf_children;


const struct dwarf_attr *dwarf_attr(enum DW_AT attr);
int dwarf_more_tags(const void *buf);
void dwarf_skip_children(const void **buf);
enum DW_TAG dwarf_tag(const void **buf);
const void *dwarf_comp_unit(void);


/* ----- General setup ----------------------------------------------------- */


void dwarf_setup(const void *debug_abbrev,const void *debug_info,
  const void *debug_info_end,const void *debug_str);

#endif /* DWARF_H */
