/*
 * dwxobj.h - Read and use objects from DWARF2 information
 *
 * Written 2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef DWXOBJ_H
#define DWXOBJ_H

#include "dwxtype.h"
#include "dwxloc.h"


enum dwx_obj_kind {
    dwx_obj_none,	/* reserved to catch uninitialized or foreign data */
    dwx_obj_unit,	/* compilation unit */
    dwx_obj_fn,		/* function */
    dwx_obj_block,	/* block */
    dwx_obj_var,	/* variable */
    dwx_obj_label,	/* label (for goto) */
};

struct dwx_obj {
    enum dwx_obj_kind kind;
    const char *name;		/* may be NULL */
    int external;		/* visible outside of enclosing unit */
    unsigned long low_pc;	/* variables: offset from enclosing low_pc */
    unsigned long high_pc;	/* not used for variables */
    union {
	struct {
	    const char *dir;		/* build directory */
	    int dir_len;		/* directory path name length
					   (without any trailing /) */
	} unit;
	struct {
	    const struct dwx_type *type;      /* type of return value */
	    const struct dwx_loc *ret_addr;   /* location of return address */
	    const struct dwx_loc *frame_addr; /* frame base address */
	} fn;
	struct {
	    const struct dwx_type *type;
	    const struct dwx_loc *loc;	/* location; may be NULL */
	    const void *constant;	/* value; NULL if variable */
	} var;
    } u;
    const struct dwx_obj *parent;	/* parent object */
    const struct dwx_obj *children;	/* child object(s) */
    struct dwx_obj *next;		/* next sibling */
};


const struct dwx_obj *dwx_find_unit(const struct dwx_obj *start,
  const char *path);
const struct dwx_obj *dwx_find_item(const struct dwx_obj *start,
  const char *name);

struct dwx_obj *dwx_read_objects(const void *unit);
  /* Calls dwxtype to read types */

#endif /* DWXOBJ_H */
