/*
 * dwxtype.h - Read types from DWARF2 information
 *
 * Written 2002,2003 by Werner Almesberger, Caltech Netlab FAST project
 */

#ifndef DWXTYPE_H
#define DWXTYPE_H

enum dwx_kind {
    dwx_none,	/* reserved to catch uninitialized or foreign data */
    dwx_uint,
    dwx_sint,
    dwx_float,
    dwx_enum,
    dwx_fn,
    dwx_array,
    dwx_struct,
    dwx_union,
    dwx_typedef,
    dwx_const,
    dwx_ptr,
    dwx_volatile,
};

struct dwx_type {
    enum dwx_kind kind;
    const char *name; /* may be NULL */
    int size; /* -1 if unknown */
    union {
	struct {
	    const struct dwx_type *type;
	} modifier;
	struct {
	    int elements; /* -1 if unspecified */
	    const struct dwx_type *type;
	    int stride_size; /* -1 if default */
	} array;
	struct {
	    struct dwx_enum_member {
		const char *name;
		int value;
		struct dwx_enum_member *next_hash; /* next collision */
		struct dwx_enum_member *next;
	    } *members;
	} _enum;
	struct {
	    struct dwx_struct_member {
		const char *name;
		int size;
		unsigned long offset;
		const struct dwx_type *type;
		struct dwx_struct_member *next;
	    } *members;
	} _struct; /* ... or union */
	struct {
	    const struct dwx_type *type;
	} _typedef;
	struct {
	    const struct dwx_type *type;
	} fn;
    } u;
    unsigned long def;		/* ELF location of definition (hash key) */
				/* (0 means no key) */
    struct dwx_type *next;	/* next hash collision */
};


/*
 * TYPE is required. If NAME is NULL, entries with any name (or no name) match.
 * If NAME is non-NULL, only entries that have a name can match. If size is -1,
 * entries with any sizes (or unknown size) match. If size is not -1, only
 * entries with that size or unknown size match. SKIP_N specifies the number of
 * matching entries to skip.
 */

const struct dwx_type *dwx_basic_type(enum dwx_kind kind,int size);
const struct dwx_type *dwx_add_ptr(const struct dwx_type *type);
const struct dwx_type *dwx_add_array(const struct dwx_type *type,
  int elements,int stride_size);
const struct dwx_type *dwx_add_fn(const struct dwx_type *type);

const struct dwx_type *dwx_index(const struct dwx_type *type,
  unsigned long *offset,int index);
const struct dwx_type *dwx_member(const struct dwx_type *type,
  unsigned long *offset,const char *name);
int dwx_lookup_enum(const char *name,int *value);

/*
 * Warning: resolve_type will clobber the attribute cache, so call it only
 * after all other attributes have been retrieved.
 */

const struct dwx_type *resolve_type(void);

const struct dwx_type *dwx_lookup_type(enum dwx_kind type,const char *name,
  int size,int skip_n);

const struct dwx_type *read_type(const void **p);
void dwx_read_types(const void *unit);

#endif /* DWXTYPE_H */
