/*
 * tstdwx.c - Retrieve various bits of data from dwarfx
 *
 * Written 2002 by Werner Almesberger, Caltech Netlab FAST project
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include "dwarfx.h"


extern void open_elf(const char *name);
extern const void *mmap_file(const char *name,int *size);

int debug = 0;


/* ----- Type name resolution ---------------------------------------------- */


static const struct dwx_type *find_type(char ***arg)
{
    const char *name = *(*arg)++;

    /* @@@ too lazy to add "int" and such */
    if (!strcmp(name,"enum") && **arg)
	return dwx_lookup_type(dwx_enum,*(*arg)++,-1,0);
    if (!strcmp(name,"struct") && **arg)
	return dwx_lookup_type(dwx_struct,*(*arg)++,-1,0);
    if (!strcmp(name,"union") && **arg)
	return dwx_lookup_type(dwx_union,*(*arg)++,-1,0);
    fprintf(stderr,"unknown type name \"%s\"\n",name);
    exit(1);
}


/* ----- Argument processing ----------------------------------------------- */


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s [-d] [-r data_file] elf_file_name "
      "[type_name ...]\n",name);
    fprintf(stderr,"%20s-d  enable debugging output\n","");
    fprintf(stderr,"%20s-r  read data from specified file and decode it\n","");
    exit(1);
}


int main(int argc,char **argv)
{
    char **arg;
    const char *data_file = NULL;
    const struct dwx_type *type;
    const void *data;
    int c;

    while ((c = getopt(argc,argv,"dr:")) != EOF)
	switch (c) {
	    case 'd':
		debug = 1;
		break;
	    case 'r':
		data_file = optarg;
		break;
	    default:
		usage(*argv);
	}
    open_elf(argv[optind]);
    if (optind+1 == argc) {
//	dump_all();
	return 0;
    }
    while (1) {
	const void *p;

	p = dwarf_comp_unit();
	if (!p) break;
	dwx_read_types(p);
    }
    arg = argv+optind+1;
    type = find_type(&arg);
    if (!type) {
	fprintf(stderr,"no such type found\n");
	exit(1);
    }
    if (*arg) {
	fprintf(stderr,"extra arguments\n");
	exit(1);
    }
    if (!data_file) {
	dwx_output_type(stdout,type);
	return 0;
    }
    data = mmap_file(data_file,NULL);
    dwx_output_data(stdout,type,data);
    return 0;
}
