/*
 * monologue.c - Send TCP segments back to yourself
 *
 * Written 2003 by Werner Almesberger
 * Copyright 2003 California Institute of Technology
 */


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>


static int snd,rcv; /* sender/receiver fds */


/* ----- Connection setup -------------------------------------------------- */


static int tcp_socket(void)
{
    int s;

    s = socket(PF_INET,SOCK_STREAM,0);
    if (s < 0) {
	perror("socket");
	exit(1);
    }
    return s;
}


static void connection(void)
{
    int s,len;
    struct sockaddr_in addr;

    s = tcp_socket();
    if (listen(s,1) < 0) {
	perror("listen");
	exit(1);
    }
    len = sizeof(addr);
    if (getsockname(s,(struct sockaddr *) &addr,&len) < 0) {
	perror("getsockname");
	exit(1);
    }
    snd = tcp_socket();
    if (connect(snd,(struct sockaddr *) &addr,sizeof(addr)) < 0) {
	perror("connect");
	exit(1);
    }
    rcv = accept(s,NULL,NULL);
    if (rcv < 0) {
	perror("accept");
	exit(1);
    }
}


/* ----- Data loop --------------------------------------------------------- */


static void loop(int segs)
{
    int i;

    for (i = 0; i < segs; i++) {
	uint32_t num;
	int wrote,got;

	num = htonl(i);
	wrote = write(snd,&num,4);
	if (wrote < 0) {
	    perror("write");
	    exit(1);
	}
	if (wrote != 4) {
	    fprintf(stderr,"short write (%d < 4)\n",wrote);
	    exit(1);
	}
	got = read(rcv,&num,4);
	if (got < 0) {
	    perror("read");
	    exit(1);
	}
	if (got != 4) {
	    fprintf(stderr,"short read (%d < 4)\n",got);
	    exit(1);
	}
    }
}


/* ----- Command-line processing ------------------------------------------- */


static void usage(const char *name)
{
    fprintf(stderr,"usage: %s num_segments\n",name);
    exit(1);
}


int main(int argc,char **argv)
{
    int segs;
    char *end;

    if (argc != 2) usage(*argv);
    segs = strtoul(argv[1],&end,0);
    if (*end) usage(*argv);
    connection();
    loop(segs);
    return 0;
}
